#!/usr/bin/python
# 

import gtk
import gobject
import gtk.glade
import os
import os.path
import select
import libuser

gladedir = "/usr/share/system-config-qemu"

# Hack for dev purposes
if os.path.exists("./system-config-qemu.glade"):
    gladedir = "."

def admin_prompt_callback(prompts, title):
    dialog = gtk.Dialog()
    dialog.set_title(title)
    dialog.set_position(gtk.WIN_POS_CENTER)
    dialog.set_icon(iconPixbuf)
    dialog.add_button(gtk.STOCK_OK, 1)
    dialog.add_button(gtk.STOCK_CANCEL, 0)

    table = gtk.Table(rows = len(prompts), columns = 2)
    dialog.vbox.pack_start(table)

    table.set_row_spacings(4)
    table.set_col_spacings(4)

    ret_list = []
    for i in range(len(prompts)):
        prompt = prompts[i]

        label = gtk.Label(prompt.prompt)
        label.set_alignment(1.0, 0.5)
        table.attach(label, 0, 1, i, i + 1)

        entry = gtk.Entry()
        entry.set_visibility(prompt.visible)
        if prompt.default_value:
            entry.set_text(prompt.default_value)
        table.attach(entry, 1, 2, i, i + 1)

        ret_list.append((prompt, entry))
        
    table.show_all()

    i = dialog.run()
    if i == 1:
        for (prompt, entry) in ret_list:
            prompt.value = entry.get_text()
        dialog.destroy()
        return True

    sys.exit(1)

class Reader:
    def run (self, fileobjs, callback=None, serviceinterval=0.1, readmax=128):
        outputs = {}
        for fileobj in fileobjs:
            outputs[fileobj] = ""
        while len (fileobjs):
            if callback:
                callback ()
            available = select.select (fileobjs, [], [], serviceinterval)
            for fileobj in available[0]:
                text = fileobj.read (readmax)
                if len (text) != 0:
                    outputs[fileobj] += text
                else:
                    #EOF
                    fileobjs.remove (fileobj)

        return outputs

class qemuAdmin:
    def __init__(self):
        self.window = gtk.glade.XML(gladedir + "/system-config-qemu.glade", "config")
        self.window.get_widget("config").show_all()
        
        userList = self.window.get_widget("accelerate-users")
        userList.set_model(gtk.ListStore(gobject.TYPE_BOOLEAN, str))
        
        user_tog = gtk.CellRendererToggle()
        user_tog.connect("toggled", self.toggle_user)
        userList.append_column(gtk.TreeViewColumn(None, user_tog, active=0))
        user_text = gtk.CellRendererText()
        userList.append_column(gtk.TreeViewColumn(None, user_text, text=1))

        self.window.signal_autoconnect({
            "on_close_clicked": self.quit,
            "on_apply_clicked": self.save,
            "on_revert_clicked": self.load,
            "on_accelerator_toggled": self.toggle_accel,
            })
        
        self.ADMIN = libuser.admin(prompt=admin_prompt_callback, prompt_data='Logon Information Required')
        self.group = self.ADMIN.lookupGroupByName("qemu")
        if self.group == None:
            cn = self.ADMIN.initGroup("qemu")
            self.ADMIN.addGroup(cn)
            self.group = self.ADMIN.lookupGroupByName("qemu")

        self.load()

    def quit(self,ignore=None):
        gtk.main_quit()

    def toggle_user(self,data,row):
        userList = self.window.get_widget("accelerate-users")
        model = userList.get_model()
        val = model.get_value(model.get_iter(int(row)), 0)
        model.set_value(model.get_iter(int(row)), 0, not val)

    def load(self,ignore=None):
        status = self.getstatusoutput("LC_ALL=C /sbin/chkconfig --list qvm86")
        if status[0] != 0:
            raise "cannot run chkconfig"

        offset = status[1].find("3:")
        if status[1][offset:offset+4] == "3:on":
            self.window.get_widget("accelerator").set_active(True)
            self.window.get_widget("accelerate-users").set_sensitive(True)
        else:
            self.window.get_widget("accelerator").set_active(False)
            self.window.get_widget("accelerate-users").set_sensitive(False)


        members = self.group.get(libuser.MEMBERNAME)
        if not members:
            members = []

        userList = self.ADMIN.enumerateUsersFull()
        userList.sort
        accelUsers = self.window.get_widget("accelerate-users").get_model()
        accelUsers.clear()
        for user in userList:
            userName = user.get(libuser.USERNAME)[0]
            uidNumber = user.get(libuser.UIDNUMBER)[0]
            if long (uidNumber) > 499L and not (userName == "nfsnobody" and (long (uidNumber) == 65534L or long (uidNumber) == 4294967294L)):
                if userName in members:
                    accelUsers.append([True,userName])
                else:
                    accelUsers.append([False,userName])

    def save(self,ignore=None):
        accelerator = self.window.get_widget("accelerator")
        if accelerator.get_active():
            off = self.getstatusoutput("LC_ALL=C /sbin/chkconfig qvm86 on")
            off = self.getstatusoutput("LC_ALL=C /sbin/service qvm86 start")
        else:
            off = self.getstatusoutput("LC_ALL=C /sbin/chkconfig qvm86 off")
            off = self.getstatusoutput("LC_ALL=C /sbin/service qvm86 stop")

        newMembers = []
        accelUsers = self.window.get_widget("accelerate-users")
        for row in accelUsers.get_model():
            if row[0] == True:
                newMembers.append(row[1])

        self.group.set(libuser.MEMBERNAME, newMembers)
        self.ADMIN.modifyGroup(self.group)

    def toggle_accel(self,widget):
        if widget.get_active():
            self.window.get_widget("accelerate-users").set_sensitive(True)
        else:
            self.window.get_widget("accelerate-users").set_sensitive(False)


    def uicallback (self):
        while gtk.events_pending ():
            if gtk.__dict__.has_key ("main_iteration"):
                gtk.main_iteration ()
            else:
                gtk.mainiteration () 

    def getstatusoutput(self,cmd):
        pipe = os.popen('{ ' + cmd + '; } 2>/dev/null', 'r')
        output = Reader().run([pipe], self.uicallback)
        text = output[pipe]
        sts = pipe.close ()
        if sts is None: sts = 0
        
        if text[-1:] == '\n': text = text[:-1]
        return sts, text


def main():
    window = qemuAdmin()
    gtk.main()

if __name__ == "__main__":
    main()
